# A C-program for MT19937: Integer version (1999/10/28)
#  genrand() generates one pseudorandom unsigned integer (32bit)
# which is uniformly distributed among 0 to 2^32-1  for each
# call. sgenrand(seed) sets initial values to the working area
# of 624 words. Before genrand(), sgenrand(seed) must be
# called once. (seed is any 32-bit integer.)
#   Coded by Takuji Nishimura, considering the suggestions by
# Topher Cooper and Marc Rieffel in July-Aug. 1997.

# This library is free software under the Artistic license:
# see the file COPYING distributed together with this code.
# For the verification of the code, its output sequence file
# mt19937int.out is attached (2001/4/2)

# Copyright (C) 1997, 1999 Makoto Matsumoto and Takuji Nishimura.
# Any feedback is very welcome. For any question, comments,
# see http://www.math.keio.ac.jp/matumoto/emt.html or email
# matumoto@math.keio.ac.jp

# REFERENCE
# M. Matsumoto and T. Nishimura,
# "Mersenne Twister: A 623-Dimensionally Equidistributed Uniform
# Pseudo-Random Number Generator",
# ACM Transactions on Modeling and Computer Simulation,
# Vol. 8, No. 1, January 1998, pp 3--30.

# Ported to bash by Michael Conrad Tilstra (Tadpol)
# Wed Nov 07 20:21:13 CST 2001
# Probably not the best port, but it works.


declare -r _mt_static_N=624
declare -r _mt_static_M=379
_mt_RAND=0

declare -a _mt_mt
_mt_mti=$(( $_mt_static_N + 1))

mt_sgenrand() {
   local seed=$1
   local part=0
   _mt_mt[0]=$(( $seed & 0xffffffff ))
   _mt_mti=1
   while [ $_mt_mti -lt $_mt_static_N ]
   do
      part=$(( 69069 * ${_mt_mt[$(( $_mt_mti - 1))]} ))
      _mt_mt[$_mt_mti]=$(( $part & 0xffffffff ))
      _mt_mti=$(( $_mt_mti + 1 ))
   done
}

mt_genrand() {
   local -i y=0
   local -i kk=0
   local -i upper=0
   local -i lower=0
   local -i idx=0
   local -i sel=0
   local -a mag #=([0]=0 [1]=0x9908b0df )

   mag[0]=0
   mag[1]=0x9908b0df

   if [ $_mt_mti -ge $_mt_static_N ];then
      
      if [ $_mt_mti -eq $(( $_mt_static_N + 1 )) ];then
         mt_sgenrand 4357
      fi

      kk=0
      while [ $kk -lt $(( $_mt_static_N - $_mt_static_M )) ];do
         upper=$(( ${_mt_mt[$kk]} & 0x80000000 ))
         lower=$(( ${_mt_mt[$(( $kk + 1))]} & 0x7fffffff))
         y=$(( $upper | $lower ))

         idx=$(($kk + $_mt_static_M ))
         sel=$(( $y & 0x1))
#         _mt_mt[$kk]=$(( ${_mt_mt[$idx]} ^ $y >> 1 ^ ${mag[$sel]} ))
         lower=$(( $y >> 1 ))
         _mt_mt[$kk]=$(( ${_mt_mt[$idx]} ^ $lower ))
         _mt_mt[$kk]=$(( ${_mt_mt[$kk]} ^ ${mag[$sel]} ))

         kk=$(( $kk + 1 ))
      done
      while [ $kk -lt $(( $_mt_static_N - 1)) ];do
         upper=$(( ${_mt_mt[$kk]} & 0x80000000 ))
         lower=$(( ${_mt_mt[$(( $kk + 1))]} & 0x7fffffff))
         y=$(( $upper | $lower ))

         idx=$(($kk + $_mt_static_M - $_mt_static_N ))
         sel=$(( $y & 0x1))
#         _mt_mt[$kk]=$(( ${_mt_mt[$idx]} ^ $y >> 1 ^ ${mag[$sel]} ))
         lower=$(( $y >> 1 ))
         _mt_mt[$kk]=$(( ${_mt_mt[$idx]} ^ $lower ))
         _mt_mt[$kk]=$(( ${_mt_mt[$kk]} ^ ${mag[$sel]} ))

         kk=$(( $kk + 1 ))
      done

      upper=$(( ${_mt_mt[$(( $_mt_static_N - 1 ))]} & 0x80000000 ))
      lower=$(( ${_mt_mt[0]} & 0x7fffffff))
      y=$(( $upper | $lower ))

      idx=$(($_mt_static_M - 1))
      sel=$(( $y & 0x1))
#_mt_mt[$(($_mt_static_N - 1))]=$(( ${_mt_mt[$idx]} ^ $y >> 1 ^ ${mag[$sel]} ))
      lower=$(( $y >> 1 ))
      _mt_mt[$(($_mt_static_N - 1))]=$(( ${_mt_mt[$idx]} ^ $lower ))
      _mt_mt[$(($_mt_static_N - 1))]=$(( ${_mt_mt[$kk]} ^ ${mag[$sel]} ))

      _mt_mti=0
   fi
   _mt_mti=$(($_mt_mti + 1))
   y=${_mt_mt[$_mt_mti]}

   y=$(( $y ^  $y >> 11 ))
   y=$(( $y ^  $y << 7 ))
   y=$(( $y & 0x9d2c5680 ))
   y=$(( $y ^  $y >> 15 ))
   y=$(( $y & 0xefc60000 ))
   y=$(( $y ^  $y << 18 ))

   _mt_RAND=$y
   return 0
}


